/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.Data;

/**
 * Klasse zum Speichern eines Wertes mit der dazugehrigen Gte. 
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class GueteWertePaar
{
	/**
	 * Wert.
	 */
	private long m_wert = -1;
	/**
	 * Gteindex.
	 */
	private float m_gueteindex = -1;
	
	
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public GueteWertePaar()
	{		
	}

	/**
	 * Konstruktor mit bernahme der Werte.
	 * 
	 * @param wert Wert.
	 * @param guete Gteindex des Wertes.
	 */
	public GueteWertePaar(long wert, float guete)
	{
		m_wert = wert;
		m_gueteindex = guete;
	}
	
	/**
	 * Konstruktor mit bernahme der Daten.
	 * 
	 * @param data Datensatz Wert.
	 */
	public GueteWertePaar(Data data)
	{
		if (data != null)
		{
			m_wert = data.getUnscaledValue("Wert").longValue();
			
			m_gueteindex = 0.0F;
			if (data.getItem("Gte").getScaledValue("Index").isNumber())
				m_gueteindex = data.getItem("Gte").getScaledValue("Index").floatValue();								
		}
		else
		{
			init();
		}
	}

	/**
	 * Initialisiert die interne Struktur auf <code>nicht ermittelbar</code>.
	 */
	private void init()
	{
		m_wert = -1;
		m_gueteindex = -1;
	}

	/**
	 * Gibt den Wert zurck.
	 * @return Wert.
	 */
	public long getWert()
	{
		return m_wert;
	}
	
	/**
	 * Setzt den Wert.
	 * @param wert zu setzender Wert.
	 */
	public void setWert(int wert)
	{
		m_wert = wert;
	}
	/**
	 * Gibt den Gteindex zurck.
	 * @return Gteindex.
	 */
	public float getGueteindex()
	{
		return m_gueteindex;
	}
	/**
	 * Setzt den Gteindex.
	 * @param gueteindex zu setzender Gteindex.
	 */
	public void setGueteindex(float gueteindex)
	{
		m_gueteindex = gueteindex;
	}

}
