/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Diese Klasse dient zum Speichern der Daten die ber die Attributgruppe 'atg.verkehrsDatenKurzZeitFs'
 * ber den Datenverteiler empfangen wurden.<p>
 * 
 *  ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.3 $ / $Date: 2015/09/02 12:49:38 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class VerkehrsDatenKurzZeitFs
{
	/**
	 * Wert Verkehrsstrke Kfz in Fz/h.
	 */
	private int m_qKfz = -1;
	/**
	 * Wert Verkehrsstrke Lkw in Fz/h.
	 */
	private int m_qLkw = -1;
	/**
	 * Wert Verkehrsstrke Pkw in Fz/h.
	 */
	private int m_qPkw = -1;
	/**
	 * Wert Geschwindigkeit Kfz in km/h.
	 */
	private int m_vKfz = -1;
	/**
	 * Wert Geschwindigkeit Lkw in km/h.
	 */
	private int m_vLkw = -1;
	/**
	 * Wert Geschwindigkeit Pkw in km/h.
	 */
	private int m_vPkw = -1;
	/**
	 * Wert Belegung in Prozent.
	 */
	private int m_b = -1;
	/**
	 * Wert Standardabweichung im km/h.
	 */
	private int m_sKfz = -1;
	/**
	 * Wert Bemessungsverkehrsstrke in PkwE/h.
	 */
	private int m_qB = -1;
	/**
	 * geglttete KFZ-Geschwindigkeit.
	 */
	private int m_vgKfz = -1;
	/**
	 * Wert und Gte Verkehrsstrke Kfz in Fz/h.
	 */
	private GueteWertePaar m_qKfzGWP;
	/**
	 * Wert und Gte Verkehrsstrke Lkw in Fz/h.
	 */
	private GueteWertePaar m_qLkwGWP;
	/**
	 * Wert und Gte Verkehrsstrke Pkw in Fz/h.
	 */
	private GueteWertePaar m_qPkwGWP;
	/**
	 * Wert und Gte Geschwindigkeit Kfz in km/h.
	 */
	private GueteWertePaar m_vKfzGWP;
	/**
	 * Wert und Gte Geschwindigkeit Lkw in km/h.
	 */
	private GueteWertePaar m_vLkwGWP;
	/**
	 * Wert und Gte Geschwindigkeit Pkw in km/h.
	 */
	private GueteWertePaar m_vPkwGWP;
	/**
	 * Wert und Gte Belegung in Prozent.
	 */
	private GueteWertePaar m_bGWP;
	/**
	 * Wert und Gte Standardabweichung im km/h.
	 */
	private GueteWertePaar m_sKfzGWP;
	/**
	 * Wert und Gte Bemessungsverkehrsstrke in PkwE/h.
	 */
	private GueteWertePaar m_qBGWP;
	/**
	 * Wert und Gte vgKfz in km/h.
	 */
	private GueteWertePaar m_vgKfzGWP;

	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public VerkehrsDatenKurzZeitFs()
	{		
	}
	
	/**
	 * Konstruktor mit bernahme der bergebenen Daten.
	 */
	public VerkehrsDatenKurzZeitFs(ResultData rdata)	
	{
		setDaten(rdata);
	}
	
	/**
	 * bernimmt die bergebenen Daten in die interne Struktur.
	 * 
	 * @param rdata Datensatz Kurzzeitdaten.
	 */
	public void setDaten(ResultData rdata)
	{
		if (rdata != null)
		{
			Data data = rdata.getData();
			if (data != null)
			{
    		m_qKfz = data.getItem("qKfz").getUnscaledValue("Wert").intValue();
    		m_qPkw = data.getItem("qPkw").getUnscaledValue("Wert").intValue();
    		m_qLkw = data.getItem("qLkw").getUnscaledValue("Wert").intValue();
    		m_vKfz = data.getItem("vKfz").getUnscaledValue("Wert").intValue();
    		m_vgKfz = data.getItem("vgKfz").getUnscaledValue("Wert").intValue();
    		m_vPkw = data.getItem("vPkw").getUnscaledValue("Wert").intValue();
    		m_vLkw = data.getItem("vLkw").getUnscaledValue("Wert").intValue();
    		m_sKfz = data.getItem("sKfz").getUnscaledValue("Wert").intValue();
    		m_qB = data.getItem("qB").getUnscaledValue("Wert").intValue();
    		m_b = data.getItem("b").getUnscaledValue("Wert").intValue();				

				m_qKfzGWP = new GueteWertePaar(data.getItem("qKfz"));
				m_qPkwGWP = new GueteWertePaar(data.getItem("qPkw"));
				m_qLkwGWP = new GueteWertePaar(data.getItem("qLkw"));
				m_vKfzGWP = new GueteWertePaar(data.getItem("vKfz"));
				m_vgKfzGWP = new GueteWertePaar(data.getItem("vgKfz"));
				m_vLkwGWP = new GueteWertePaar(data.getItem("vLkw"));
				m_vPkwGWP = new GueteWertePaar(data.getItem("vPkw"));
				m_sKfzGWP = new GueteWertePaar(data.getItem("sKfz"));
				m_qBGWP = new GueteWertePaar(data.getItem("qB"));
				m_bGWP = new GueteWertePaar(data.getItem("b"));
			}
			else
			{
				init();
			}
		}
	}
	
	/**
	 * Initialisiert die interne Struktur auf <code>nicht ermittelbar</code>.
	 */
	public void init()
	{
		m_qKfz = -1;
		m_qPkw = -1;
		m_qLkw = -1;
		m_vKfz = -1;
		m_vgKfz = -1;
		m_vPkw = -1;
		m_vLkw = -1;
		m_sKfz = -1;
		m_qB = -1;
		m_b = -1;						

		m_qKfzGWP = new GueteWertePaar();
		m_qPkwGWP = new GueteWertePaar();
		m_qLkwGWP = new GueteWertePaar();
		m_vKfzGWP = new GueteWertePaar();
		m_vgKfzGWP = new GueteWertePaar();
		m_vLkwGWP = new GueteWertePaar();
		m_vPkwGWP = new GueteWertePaar();
		m_sKfzGWP = new GueteWertePaar();
		m_qBGWP = new GueteWertePaar();
		m_bGWP = new GueteWertePaar();	
	}

	/**
	 * Gibt zurck, ob aktuelle Daten empfangen wurden.
	 * 
	 * @return true wenn Verkehrsdaten vorhanden, ansonsten false;
	 */
	public boolean hasDatenErhalten()
	{
		if ((m_qKfz == -1) &&
				(m_qPkw == -1) &&
				(m_qLkw == -1) &&
				(m_vKfz == -1) &&
				(m_vgKfz == -1) &&
				(m_vPkw == -1) &&
				(m_vLkw == -1) &&
				(m_sKfz == -1) &&
				(m_qB == -1) &&
				(m_b == -1))
		{
			return false;
		}
		
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer string = new StringBuffer();
		
		string.append(
				"\n qKfz  : " + m_qKfz + " Fz/h " +
				"\n qPkw  : " + m_qPkw + " Fz/h " +
				"\n qLkw  : " + m_qLkw + " Fz/h " +
				"\n vKfz  : " + m_vKfz + " km/h " +
				"\n vgKfz  : " + m_vgKfz + " km/h " +
				"\n vPkw  : " + m_vPkw + " km/h " +
				"\n vLkw  : " + m_vLkw + " km/h " +
				"\n sKfz  : " + m_sKfz + " km/h " +
				"\n qB    : " + m_qB + " Fz/h " +
				"\n B     : " + m_b + " % ");
		
		return string.toString();
	}

	/**
	 * Gibt die Verkehrsstrke Kfz in Fz/h zurck.
	 * 
	 * @return Verkehrsstrke Kfz in Fz/h.
	 */
	public int getQKfz()
	{
		return m_qKfz;
	}

	/**
	 * Setzt die Verkehrsstrke Kfz in Fz/h.
	 * 
	 * @param kfz zu setzende Verkehrsstrke Kfz in Fz/h.
	 */
	public void setQKfz(int kfz)
	{
		m_qKfz = kfz;
	}

	/**
	 * Gibt die Verkehrsstrke Lkw in Fz/h zurck.
	 * 
	 * @return Verkehrsstrke Lkw in Fz/h.
	 */
	public int getQLkw()
	{
		return m_qLkw;
	}

	/**
	 * Setzt die Verkehrsstrke Lkw in Fz/h.
	 * 
	 * @param lkw zu setzende Verkehrsstrke Lkw in Fz/h.
	 */
	public void setQLkw(int lkw)
	{
		m_qLkw = lkw;
	}

	/**
	 * Gibt die Verkehrsstrke Pkw in Fz/h zurck.
	 * 
	 * @return Verkehrsstrke Pkw in Fz/h.
	 */
	public int getQPkw()
	{
		return m_qPkw;
	}

	/**
	 * Setzt die Verkehrsstrke Pkw in Fz/h.
	 * 
	 * @param pkw zu setzende Verkehrsstrke Pkw in Fz/h.
	 */
	public void setQPkw(int pkw)
	{
		m_qPkw = pkw;
	}

	/**
	 * Gibt die Geschwindigkeit Kfz in km/h zurck.
	 * 
	 * @return Geschwindigkeit Kfz in km/h.
	 */
	public int getVKfz()
	{
		return m_vKfz;
	}

	/**
	 * Setzt die Geschwindigkeit Kfz in km/h.
	 * 
	 * @param kfz zu setzende Geschwindigkeit Kfz in km/h.
	 */
	public void setVKfz(int kfz)
	{
		m_vKfz = kfz;
	}

	/**
	 * Gibt die Geschwindigkeit Lkw in km/h zurck.
	 * 
	 * @return Geschwindigkeit Lkw in km/h.
	 */
	public int getVLkw()
	{
		return m_vLkw;
	}

	/**
	 * Setzt die Geschwindigkeit Lkw in km/h.
	 * 
	 * @param lkw zu setzende Geschwindigkeit Lkw in km/h.
	 */
	public void setVLkw(int lkw)
	{
		m_vLkw = lkw;
	}

	/**
	 * Gibt die Geschwindigkeit Pkw in km/h zurck.
	 * 
	 * @return Geschwindigkeit Pkw in km/h.
	 */
	public int getVPkw()
	{
		return m_vPkw;
	}

	/**
	 * Setzt die Geschwindigkeit Pkw in km/h.
	 * 
	 * @param pkw zu setzende Geschwindigkeit Pkw in km/h.
	 */
	public void setVPkw(int pkw)
	{
		m_vPkw = pkw;
	}

	/**
	 * Gibt die Belegung in Prozent zurck.
	 * 
	 * @return the Belegung in Prozent.
	 */
	public int getB()
	{
		return m_b;
	}

	/**
	 * Setzt die Belegung in Prozent.
	 * 
	 * @param b zu setzende Belegung in Prozent.
	 */
	public void setB(int b)
	{
		m_b = b;
	}

	/**
	 * Gibt die Standardabweichung im km/h zurck.
	 * 
	 * @return Standardabweichung im km/h.
	 */
	public int getSKfz()
	{
		return m_sKfz;
	}

	/**
	 * Setzt die Standardabweichung im km/h.
	 * 
	 * @param kfz zu setzende Standardabweichung im km/h.
	 */
	public void setSKfz(int kfz)
	{
		m_sKfz = kfz;
	}

	/**
	 * Gibt die Bemessungsverkehrsstrke in PkwE/h zurck.
	 * 
	 * @return Bemessungsverkehrsstrke in PkwE/h.
	 */
	public int getQB()
	{
		return m_qB;
	}

	/**
	 * Setzt die Bemessungsverkehrsstrke in PkwE/h.
	 * 
	 * @param qb zu setzende Bemessungsverkehrsstrke in PkwE/h.
	 */
	public void setQB(int qb)
	{
		m_qB = qb;
	}
	

	/**
	 * Gibt den Wert der Variable m_vgKfz zurck.
	 *
	 * @return Wert der Variable m_vgKfz.
	 */
	public int getVgKfz()
	{
		return m_vgKfz;
	}

	/**
	 * Setzt den Wert der Variable m_vgKfz.
	 *
	 * @param vgKfz zu setzender Wert der Variable m_vgKfz.
	 */
	public void setVgKfz(int vgKfz)
	{
		m_vgKfz = vgKfz;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Kfz zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Kfz.
	 */
	public float getQKfzGte()
	{
		if (m_qKfzGWP != null)
		{
			return m_qKfzGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Lkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Lkw.
	 */
	public float getQLkwGte()
	{
		if (m_qLkwGWP != null)
		{
			return m_qLkwGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Pkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Pkw.
	 */
	public float getQPkwGte()
	{
		if (m_qPkwGWP != null)
		{
			return m_qPkwGWP.getGueteindex();
		}
	
		return 0;
	}

	
	/**
	 * Gibt den Gteindex der vKfz zurck.
	 * 
	 * @return Gteindex vKfz.
	 */
	public float getVKfzGte()
	{
		if (m_vKfzGWP != null)
		{
			return m_vKfzGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der vLkw zurck.
	 * 
	 * @return Gteindex vLkw.
	 */
	public float getVLkwGte()
	{
		if (m_vLkwGWP != null)
		{
			return m_vLkwGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Pkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Pkw.
	 */
	public float getVPkwGte()
	{
		if (m_vPkwGWP != null)
		{
			return m_vPkwGWP.getGueteindex();
		}
	
		return 0;
	}
	
	/**
	 * Gibt den Gteindex der sKfz zurck.
	 * 
	 * @return Gteindex sKfz.
	 */
	public float getSKfzGte()
	{
		if (m_sKfzGWP != null)
		{
			return m_sKfzGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der qB zurck.
	 * 
	 * @return Gteindex qB.
	 */
	public float getQBGte()
	{
		if (m_qBGWP != null)
		{
			return m_qBGWP.getGueteindex();
		}
	
		return 0;
	}
	/**
	 * Gibt den Gteindex der B zurck.
	 * 
	 * @return Gteindex B.
	 */
	public float getBGte()
	{
		if (m_bGWP != null)
		{
			return m_bGWP.getGueteindex();
		}
	
		return 0;
	}
	/**
	 * Gibt den Gteindex der B zurck.
	 * 
	 * @return Gteindex B.
	 */
	public float getVgKfzGte()
	{
		if (m_vgKfzGWP != null)
		{
			return m_vgKfzGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Wert der Variable m_vgKfzGWP zurck.
	 *
	 * @return Wert der Variable m_vgKfzGWP.
	 */
	public GueteWertePaar getVgKfzGWP()
	{
		return m_vgKfzGWP;
	}

	/**
	 * Setzt den Wert der Variable m_vgKfzGWP.
	 *
	 * @param vgKfzGWP zu setzender Wert der Variable m_vgKfzGWP.
	 */
	public void setVgKfzGWP(GueteWertePaar vgKfzGWP)
	{
		m_vgKfzGWP = vgKfzGWP;
	}
	
}
