/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Diese Klasse dient zum Speichern der Daten die ber die Attributgruppe 'atg.verkehrsDatenKurzZeitTrendExtraPolationMq'
 * ber den Datenverteiler empfangen wurden.<p>
 * 
 *  ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.3 $ / $Date: 2014/01/31 09:33:45 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class VerkehrsDatenKurzZeitTrendExtraPolationMq
{
	/**
	 * Wert Prognoseverkehrsstrke Kfz in Fz/h.
	 */
	private int m_qKfzP = -1;
	/**
	 * Wert Prognoseverkehrsstrke Lkw in Fz/h.
	 */
	private int m_qLkwP = -1;
	/**
	 * Wert Prognoseverkehrsstrke Pkw in Fz/h.
	 */
	private int m_qPkwP = -1;
	/**
	 * Wert Prognosegeschwindigkeit Kfz in km/h.
	 */
	private int m_vKfzP = -1;
	/**
	 * Wert Prognosegeschwindigkeit Lkw in km/h.
	 */
	private int m_vLkwP = -1;
	/**
	 * Wert Prognosegeschwindigkeit Pkw in km/h.
	 */
	private int m_vPkwP = -1;
	/**
	 * Wert Prognosebemessungsverkehrsstrke in PkwE/h.
	 */
	private int m_qBP = -1;
	/**
	 * Wert Prognosedichte Kfz in Fz/km.
	 */
	private int m_kKfzP = -1;
	/**
	 * Wert Lkw-Anteil in Prozent.
	 */
	private int m_aLkwP = -1;

	/**
	 * Wert und Gte Verkehrsstrke Kfz in Fz/h.
	 */
	private GueteWertePaar m_qKfzPGWP;
	/**
	 * Wert und Gte Verkehrsstrke Lkw in Fz/h.
	 */
	private GueteWertePaar m_qLkwPGWP;
	/**
	 * Wert und Gte Verkehrsstrke Pkw in Fz/h.
	 */
	private GueteWertePaar m_qPkwPGWP;
	/**
	 * Wert und Gte Geschwindigkeit Kfz in km/h.
	 */
	private GueteWertePaar m_vKfzPGWP;
	/**
	 * Wert und Gte Geschwindigkeit Lkw in km/h.
	 */
	private GueteWertePaar m_vLkwPGWP;
	/**
	 * Wert und Gte Geschwindigkeit Pkw in km/h.
	 */
	private GueteWertePaar m_vPkwPGWP;
	/**
	 * Wert und Gte Standardabweichung im km/h.
	 */
	private GueteWertePaar m_kKfzPGWP;
	/**
	 * Wert und Gte Bemessungsverkehrsstrke in PkwE/h.
	 */
	private GueteWertePaar m_qBPGWP;
	/**
	 * Wert und Gte Lkw-Anteil in Prozent.
	 */
	private GueteWertePaar m_aLkwPGWP;

	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public VerkehrsDatenKurzZeitTrendExtraPolationMq()
	{		
	}
	
	/**
	 * Konstruktor mit bernahme der bergebenen Daten.
	 */
	public VerkehrsDatenKurzZeitTrendExtraPolationMq(ResultData rdata)	
	{
		setDaten(rdata);
	}
	
	/**
	 * bernimmt die bergebenen Daten in die interne Struktur.
	 * 
	 * @param rdata Datensatz Kurzzeitdaten.
	 */
	public void setDaten(ResultData rdata)
	{
		if (rdata != null)
		{
			Data data = rdata.getData();
			if (data != null)
			{
    		m_qKfzP = data.getItem("QKfzP").getUnscaledValue("Wert").intValue();
    		m_qPkwP = data.getItem("QPkwP").getUnscaledValue("Wert").intValue();
    		m_qLkwP = data.getItem("QLkwP").getUnscaledValue("Wert").intValue();
    		m_vKfzP = data.getItem("VKfzP").getUnscaledValue("Wert").intValue();
    		m_vPkwP = data.getItem("VPkwP").getUnscaledValue("Wert").intValue();
    		m_vLkwP = data.getItem("VLkwP").getUnscaledValue("Wert").intValue();
    		m_qBP = data.getItem("QBP").getUnscaledValue("Wert").intValue();				
    		m_kKfzP = data.getItem("KKfzP").getUnscaledValue("Wert").intValue();
    		m_aLkwP = data.getItem("ALkwP").getUnscaledValue("Wert").intValue();

    		m_qKfzPGWP = new GueteWertePaar(data.getItem("QKfzP"));
				m_qPkwPGWP = new GueteWertePaar(data.getItem("QPkwP"));
				m_qLkwPGWP = new GueteWertePaar(data.getItem("QLkwP"));
				m_vKfzPGWP = new GueteWertePaar(data.getItem("VKfzP"));
				m_vLkwPGWP = new GueteWertePaar(data.getItem("VLkwP"));
				m_vPkwPGWP = new GueteWertePaar(data.getItem("VPkwP"));
				m_kKfzPGWP = new GueteWertePaar(data.getItem("KKfzP"));
				m_qBPGWP = new GueteWertePaar(data.getItem("QBP"));
				m_aLkwPGWP = new GueteWertePaar(data.getItem("ALkwP"));
			}
			else
			{
				init();
			}
		}
	}

	/**
	 * Initialisiert die interne Struktur auf <code>nicht ermittelbar</code>.
	 */
	public void init()
	{
		m_qKfzP = -1;
		m_qPkwP = -1;
		m_qLkwP = -1;
		m_vKfzP = -1;
		m_vPkwP = -1;
		m_vLkwP = -1;
		m_qBP = -1;
		m_kKfzP = -1;
		m_aLkwP = -1;
		
		m_qKfzPGWP = new GueteWertePaar();
		m_qPkwPGWP = new GueteWertePaar();
		m_qLkwPGWP = new GueteWertePaar();
		m_vKfzPGWP = new GueteWertePaar();
		m_vLkwPGWP = new GueteWertePaar();
		m_vPkwPGWP = new GueteWertePaar();
		m_kKfzPGWP = new GueteWertePaar();
		m_qBPGWP = new GueteWertePaar();
		m_aLkwPGWP = new GueteWertePaar();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer string = new StringBuffer();
		
		string.append(
				"\n qKfzP  : " + m_qKfzP + " Fz/h " +
				"\n qPkwP  : " + m_qPkwP + " Fz/h " +
				"\n qLkwP  : " + m_qLkwP + " Fz/h " +
				"\n vKfzP  : " + m_vKfzP + " km/h " +
				"\n vPkwP  : " + m_vPkwP + " km/h " +
				"\n vLkwP  : " + m_vLkwP + " km/h " +
				"\n kKfzP  : " + m_kKfzP + " Fz/km " +
				"\n aLkwP  : " + m_aLkwP + " % " +
				"\n qBP    : " + m_qBP + " Fz/h ");
		
		return string.toString();
	}

	/**
	 * Gibt zurck, ob aktuelle Daten empfangen wurden.
	 * 
	 * @return true wenn Verkehrsdaten vorhanden, ansonsten false;
	 */
	public boolean hasDatenErhalten()
	{
		if ((m_qKfzP == -1) &&
				(m_qPkwP == -1) &&
				(m_qLkwP == -1) &&
				(m_vKfzP == -1) &&
				(m_vPkwP == -1) &&
				(m_vLkwP == -1) &&
				(m_kKfzP == -1) &&
				(m_aLkwP == -1) &&
				(m_qBP == -1))
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Gibt die Prognoseverkehrsstrke Kfz in Fz/h zurck.
	 * 
	 * @return Prognoseverkehrsstrke Kfz in Fz/h.
	 */
	public int getQKfzP()
	{
		return m_qKfzP;
	}

	/**
	 * Setzt die Prognoseverkehrsstrke Kfz in Fz/h.
	 * 
	 * @param kfz zu setzende Prognoseverkehrsstrke Kfz in Fz/h.
	 */
	public void setQKfzP(int kfz)
	{
		m_qKfzP = kfz;
	}

	/**
	 * Gibt die Prognoseverkehrsstrke Lkw in Fz/h zurck.
	 * 
	 * @return Prognoseverkehrsstrke Lkw in Fz/h.
	 */
	public int getQLkwP()
	{
		return m_qLkwP;
	}

	/**
	 * Setzt die Prognoseverkehrsstrke Lkw in Fz/h.
	 * 
	 * @param lkw zu setzende Prognoseverkehrsstrke Lkw in Fz/h.
	 */
	public void setQLkwP(int lkw)
	{
		m_qLkwP = lkw;
	}

	/**
	 * Gibt die Prognoseverkehrsstrke Pkw in Fz/h zurck.
	 * 
	 * @return Prognoseverkehrsstrke Pkw in Fz/h.
	 */
	public int getQPkwP()
	{
		return m_qPkwP;
	}

	/**
	 * Setzt die Prognoseverkehrsstrke Pkw in Fz/h.
	 * 
	 * @param pkw zu setzende Prognoseverkehrsstrke Pkw in Fz/h.
	 */
	public void setQPkwP(int pkw)
	{
		m_qPkwP = pkw;
	}

	/**
	 * Gibt die Prognosegeschwindigkeit Kfz in km/h zurck.
	 * 
	 * @return Prognosegeschwindigkeit Kfz in km/h.
	 */
	public int getVKfzP()
	{
		return m_vKfzP;
	}

	/**
	 * Setzt die Prognosegeschwindigkeit Kfz in km/h.
	 * 
	 * @param kfz zu setzende Prognosegeschwindigkeit Kfz in km/h.
	 */
	public void setVKfzP(int kfz)
	{
		m_vKfzP = kfz;
	}

	/**
	 * Gibt die Prognosegeschwindigkeit Lkw in km/h zurck.
	 * 
	 * @return Prognosegeschwindigkeit Lkw in km/h.
	 */
	public int getVLkwP()
	{
		return m_vLkwP;
	}

	/**
	 * Setzt die Prognosegeschwindigkeit Lkw in km/h.
	 * 
	 * @param lkw zu setzende Prognosegeschwindigkeit Lkw in km/h.
	 */
	public void setVLkwP(int lkw)
	{
		m_vLkwP = lkw;
	}

	/**
	 * Gibt die Prognosegeschwindigkeit Pkw in km/h zurck.
	 * 
	 * @return Prognosegeschwindigkeit Pkw in km/h.
	 */
	public int getVPkwP()
	{
		return m_vPkwP;
	}

	/**
	 * Setzt die Prognosegeschwindigkeit Pkw in km/h.
	 * 
	 * @param pkw zu setzende Prognosegeschwindigkeit Pkw in km/h.
	 */
	public void setVPkwP(int pkw)
	{
		m_vPkwP = pkw;
	}

	/**
	 * Gibt die Prognosebemessungsverkehrsstrke in PkwE/h zurck.
	 * 
	 * @return Prognosebemessungsverkehrsstrke in PkwE/h.
	 */
	public int getQBP()
	{
		return m_qBP;
	}

	/**
	 * Setzt die Prognosebemessungsverkehrsstrke in PkwE/h.
	 * 
	 * @param qb zu setzende Prognosebemessungsverkehrsstrke in PkwE/h.
	 */
	public void setQBP(int qb)
	{
		m_qBP = qb;
	}

	/**
	 * Gibt den Wert Prognosedichte Kfz in Fz/km zurck.
	 * @return Wert Prognosedichte Kfz in Fz/km.
	 */
	public int getKKfzP()
	{
		return m_kKfzP;
	}

	/**
	 * Setzt den Wert Prognosedichte Kfz in Fz/km.
	 * @param kfzP zu setzender Wert Prognosedichte Kfz in Fz/km.
	 */
	public void setKKfzP(int kfzP)
	{
		m_kKfzP = kfzP;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Kfz zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Kfz.
	 */
	public float getQKfzPGte()
	{
		if (m_qKfzPGWP != null)
		{
			return m_qKfzPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Lkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Lkw.
	 */
	public float getQLkwPGte()
	{
		if (m_qLkwPGWP != null)
		{
			return m_qLkwPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Pkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Pkw.
	 */
	public float getQPkwPGte()
	{
		if (m_qPkwPGWP != null)
		{
			return m_qPkwPGWP.getGueteindex();
		}
	
		return 0;
	}

	
	/**
	 * Gibt den Gteindex der vKfz zurck.
	 * 
	 * @return Gteindex vKfz.
	 */
	public float getVKfzPGte()
	{
		if (m_vKfzPGWP != null)
		{
			return m_vKfzPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der vLkw zurck.
	 * 
	 * @return Gteindex vLkw.
	 */
	public float getVLkwPGte()
	{
		if (m_vLkwPGWP != null)
		{
			return m_vLkwPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der Verkehrsstrke Pkw zurck.
	 * 
	 * @return Gteindex Verkehrsstrke Pkw.
	 */
	public float getVPkwPGte()
	{
		if (m_vPkwPGWP != null)
		{
			return m_vPkwPGWP.getGueteindex();
		}
	
		return 0;
	}
	
	/**
	 * Gibt den Gteindex der kKfz zurck.
	 * 
	 * @return Gteindex kKfz.
	 */
	public float getKKfzPGte()
	{
		if (m_kKfzPGWP != null)
		{
			return m_kKfzPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Gteindex der qB zurck.
	 * 
	 * @return Gteindex qB.
	 */
	public float getQBGte()
	{
		if (m_qBPGWP != null)
		{
			return m_qBPGWP.getGueteindex();
		}
	
		return 0;
	}

	/**
	 * Gibt den Wert der Variable m_aLkwP zurck.
	 *
	 * @return Wert der Variable m_aLkwP.
	 */
	public int getALkwP()
	{
		return m_aLkwP;
	}

	/**
	 * Setzt den Wert der Variable m_aLkwP.
	 *
	 * @param aLkwP zu setzender Wert der Variable m_aLkwP.
	 */
	public void setALkwP(int aLkwP)
	{
		m_aLkwP = aLkwP;
	}

	/**
	 * Gibt den Wert der Variable m_aLkwPGWP zurck.
	 *
	 * @return Wert der Variable m_aLkwPGWP.
	 */
	public GueteWertePaar getALkwPGWP()
	{
		return m_aLkwPGWP;
	}
}
