/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.Ereignis;
import de.bsvrz.ste.stealgsba.algorithmen.lib.MessQuerschnitt;
import de.bsvrz.ste.stealgsba.algorithmen.lib.ParameterMqZuordnung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.PassivierungFahrstreifen;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.UmfeldDatenSensor;
import de.bsvrz.ste.stealgsba.algorithmen.lib.VerkehrsDatenKurzZeitTrendExtraPolationMq;
import de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag.ParameterLkwUeberholverbotSichtWochentag.ParameterLkwUeberholverbotSichtWochentagFahrstreifen;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.UrsacheneinheitErweiterung;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Klasse implementiert die Funktionalitt der Situationsbearbeitung fr einen Algorithmus.<p>
 * Verwaltung der fr die Situationsbestimmung notwendigen Parameter und Eingangswerte.
 *
 * @version $Revision: 1.1 $ / $Date: 2015/08/26 13:35:37 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class LkwUeberholverbotSichtWochentagUe extends UrsacheneinheitErweiterung implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static Debug m_debug = Debug.getLogger();
	/**
	 * Systemobjekt der Ursacheneinheit.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Empfang der Parameter Lkw-berholverbot Sicht.
	 */
	private AttributeGroup m_atgParameter;
	/**
	 * Attributgruppe fr den Empfang der Parameter Zuordnung Messquerschnitt.
	 */
	private AttributeGroup m_atgParameterMqZuordnung;
	/**
	 * Attributgruppe fr den Empfang der Parameter Zuordnung Umfelddatensensor.
	 */
	private AttributeGroup m_atgParameterUfdsZuordnung;
	/**
	 * Attributgruppe fr den Versand der Analysedaten der Situationsbestimmung.
	 */
	private AttributeGroup m_atgAnalyse;
	/**
	 * Aspekt fr den Empfang der globalen Parameter.
	 */
	private Aspect m_aspParameter;
	/**
	 * Aspekt fr den Versand der Analysedaten.
	 */
	private Aspect m_aspAnalyse;
  /**
   * Parameter fr die Situationsbestimmung Lkw-berholverbot Sichtbehinderungen.
   */
  private ParameterLkwUeberholverbotSichtWochentag m_parameter;  
  /**
   * Parameter Zuordnung Messquerschnitt.
   */
  private ParameterMqZuordnung m_parameterMqZuordnung;  
  /**
   * Parameter Zuordnung Umfelddatensensor.
   */
  private ParameterUfdsZuordnung m_parameterUfdsZuordnung;  
  /**
   * Zuordnung Messquerschnitt Alt.
   */
  private List<SystemObject> m_mqZuordnungAlt = new ArrayList<SystemObject>();  
  /**
   * Zuordnung Umfelddatensensor Alt.
   */
  private List<SystemObject> m_ufdsZuordnungAlt = new ArrayList<SystemObject>();  
  /**
   * Speicher fr die Objekte zur Verwaltung der Messquerschnittsdaten.<br>
   * Key ist die PID des Messquerschnitt.
   */
  private HashMap<String, MessQuerschnitt> m_mapMessQuerschnitte = new HashMap<String, MessQuerschnitt>();
  /**
   * Liste der passivierten Fahrstreifen.
   */
  private PassivierungFahrstreifen m_passivierungFahrStreifen;
  /**
   * Speicher fr die Objekte zur Verwaltung der Umfelddatensensordaten.<br>
   * Key ist die PID der Umfelddatensensor.
   */
  private HashMap<String, UmfeldDatenSensor> m_mapUmfeldDatenSensor = new HashMap<String, UmfeldDatenSensor>();
  /**
   * Aspekt fr den Empfang der Prognosedaten.
   */
  private Aspect m_aspektPrognose;
	/**
	 * In dieser HashMap sind die zu bercksichtigenden Ereignisse abgelegt.
	 * Key ist die Pid des Ereignisses.
	 */
	private Map<String, Ereignis> m_mapEreignis =  Collections.synchronizedMap(new HashMap<String, Ereignis>());
  /**
   * Kennung, ob bei der Ereignisbestimmung ein Fehler aufgetreten ist.
   */
	private boolean m_fehlerEreignis;
	/**
	 * Kennung, ob die Betriebsmeldung schon versenndet wurde.
	 */
	private boolean m_betrmVersendet;

	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * Anmelden zum Empfang der bentigten Parameter am Datenverteiler.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param ue Ursacheneinheit.
	 * @param aspektVersand Aspekt fr den Versand.
	 */
	public LkwUeberholverbotSichtWochentagUe(ClientDavInterface connection, SystemObject ue, Aspect aspektVersand, String aspektPrognose)
	{
		super(connection, ue, aspektVersand);

		m_objekt = ue;

		m_aspektPrognose = m_dataModel.getAspect(aspektPrognose);
		
		m_atgParameter = m_dataModel.getAttributeGroup("atg.sswSbaUeLkwberholverbotSichtWochentag");
		m_atgParameterMqZuordnung = m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung");
		m_atgParameterUfdsZuordnung = m_dataModel.getAttributeGroup("atg.sswSbaUeUfdsZuordnung");

		m_aspParameter = m_dataModel.getAspect("asp.parameterSoll");

		m_atgAnalyse = m_dataModel.getAttributeGroup("atg.sswSbaUeSituationAnalyseMq");
		m_aspAnalyse = m_dataModel.getAspect("asp.analyse");

		// Daten zum Empfang anmelden
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterMqZuordnung, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameterUfdsZuordnung, m_aspParameter);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameter, m_aspParameter);

		// Daten zum Versand anmelden
		quelleAnmeldenObjekt(this, m_objekt, m_atgAnalyse, m_aspAnalyse);
	}
	
	/**
	 * Diese Methode berechnet die aktuelle Situation der Ursacheneinheit entsprechend den
	 * empfangenen Parametern und versendet die Analysedaten ber den Datenverteiler..
	 */
	public boolean berechnen()
	{
		boolean _statusAlt = this.getStatus();
		boolean zurueck = false;
    boolean stoerungMesswerte = true;
	  boolean einschaltKriterium = false;
    boolean ausschaltKriterium = false;
    int ausschaltZaehlerMq;
    
    SituationAnalyseMq analyse = new SituationAnalyseMq();

    if (m_parameter == null)
		{
//			System.out.println(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");
			m_debug.fine(m_objekt.getPid() + " Noch keine Parameter erhalten. Abbruch!");

			return false;
		}

    /**
     * Wenn kein zu prfendes Ereignis vorhanden, Betriebsmeldung versenden und Berechnung abbrechen.
     */
    if (m_fehlerEreignis)
    {
    	MessageState state = MessageState.NEW_MESSAGE;
    	if (m_betrmVersendet)
    	{
    		state = MessageState.REPEAT_MESSAGE;
    	}
    	
			String nachricht =  "Zu dem EreignisTyp='" + m_parameter.getEreignisTyp() + "' konnte kein zu prfendes Ereignis gefunden werden. Berechnung wird abgebrochen!";

			MessageSender.getInstance().sendMessage(this.getClass().getSimpleName(), MessageType.APPLICATION_DOMAIN, " ", MessageGrade.ERROR, null, state, new MessageCauser(null, "", ""), nachricht);
    	
    	m_betrmVersendet = true;
    	
    	return false;
    }
    
    if (m_betrmVersendet)
    {
			String nachricht =  "Zu dem EreignisTyp='" + m_parameter.getEreignisTyp() + "' konnte ein zu prfendes Ereignis gefunden werden. Berechnung wird fortgesetzt!";

			MessageSender.getInstance().sendMessage(this.getClass().getSimpleName(), MessageType.APPLICATION_DOMAIN, " ", MessageGrade.ERROR, null, MessageState.GOOD_MESSAGE, new MessageCauser(null, "", ""), nachricht);

			m_betrmVersendet = false;
    }

    ausschaltZaehlerMq = 0;
    
    Object[] keys = m_mapMessQuerschnitte.keySet().toArray();
    for (int i = 0; i < keys.length; i++)
		{
			String pidMq = (String)keys[i];
			
			MessQuerschnitt mq = m_mapMessQuerschnitte.get(pidMq);
			if (mq != null)
			{
			  boolean einschaltKriteriumMq = false;

				int anzFs = bestimmeAnzFahrstreifenAktuell(mq);
				ParameterLkwUeberholverbotSichtWochentagFahrstreifen parameter = m_parameter.getParameterFahrstreifen(anzFs);
				
				if (parameter == null)
					return false;

		    System.out.println(m_objekt.getPid() + " -> Parameter : " + pidMq +  " anzFs " + anzFs + " " + parameter + " Status Alt " + _statusAlt);
		    m_debug.fine(m_objekt.getPid() + " -> Parameter : " + pidMq + " anzFs " + anzFs + " " + parameter + " Status Alt " + _statusAlt);

		    VerkehrsDatenKurzZeitTrendExtraPolationMq prognoseDaten = mq.getVerkehrsDatenKurzZeitTrendExtraPolationMq();
				
//				System.out.println("VerkehrsDaten Prognose " + pidMq + " " + prognoseDaten);
				
				if (prognoseDaten == null)
				{
					ausschaltZaehlerMq++;
					continue;
				}
				
		    Object[] keysUfdm = m_mapUmfeldDatenSensor.keySet().toArray();
		    for (int j = 0; j < keysUfdm.length; j++)
				{
		    	String pidUfds = (String)keysUfdm[j];

		    	UmfeldDatenSensor ufds = m_mapUmfeldDatenSensor.get(pidUfds);		    	
		    	if (ufds != null)
		    	{
		    		// Parameter
				    int _qlkwEin = parameter.getQLkwPEin();
				    int _qlkwAus = parameter.getQLkwPAus();
				    int _qbEin = parameter.getQBUevPEin();
				    int _qbAus = parameter.getQBUevPAus();
				    int _swEin = parameter.getSichtStufeEin();
				    int _swAus = parameter.getSichtStufeAus();
				    	
						if (prognoseDaten.hasDatenErhalten())
						{
							stoerungMesswerte = false;

							// Messwerte
							int _qb = prognoseDaten.getQBP();
							int _qLkwP = prognoseDaten.getQLkwP();
							int _swStufe = ufds.getSichtStufe();

							System.out.println(m_objekt.getPid() + " -> Messwerte : " + mq.getPid() + " QB " + _qb + " QLkwP " + _qLkwP + " SW " + _swStufe);
							System.out.println(m_objekt.getPid() + " -> Ereignis  : " + m_parameter.getEreignisTyp() + " gltig " + isEreignisGltig());
							
							if ((_qb >= 0) && (_qLkwP >= 0))
							{
								if ((_qb > _qbEin) && (_qLkwP > _qlkwEin) && (_swStufe > _swEin) && isEreignisGltig())
									einschaltKriteriumMq = true;
			
								if ((_qb < _qbAus) || (_qLkwP < _qlkwAus) || (_swStufe <= _swEin) || !isEreignisGltig())
									ausschaltZaehlerMq++;
							}
							else
							{
								ausschaltZaehlerMq++;
							}
						}
						else
						{
							ausschaltZaehlerMq++;
						}
						
						einschaltKriterium |= einschaltKriteriumMq;
							
						analyse.addAnalyseMq(mq.getObjekt(), einschaltKriteriumMq);		    		
		    	}
				}
			}
		}

    if (ausschaltZaehlerMq == (keys.length * m_mapUmfeldDatenSensor.size()))
    {
    	ausschaltKriterium = true;
    }

    analyse.setEinschaltkriterium(einschaltKriterium);
    analyse.setAusschaltkriterium(ausschaltKriterium);
    
    System.out.println(m_objekt.getPid() + " -> Analyse   : " + analyse);
    
    // Analysedaten versenden.
    sendeAnalyseDaten(analyse);

    /**
     * Wenn mit Nachlaufzeit, nur einschaltkriterium beachten.
     */
    if (isMindeststandzeitAlsNachlaufzeit())
    {
    	zurueck = einschaltKriterium;
    }
    else
    {
    	zurueck = bestimmeKriteriumMq(einschaltKriterium, ausschaltKriterium, _statusAlt);
    }

    // Keine Messwerte vorhanden, Schaltgund deaktivieren 
    if (stoerungMesswerte)
    	zurueck = false;

    System.out.println(m_objekt.getPid() + " -> Akt Status: " + zurueck);
    
    return zurueck;
	}

	/**
	 * Diese Methode bestimmt die Anzahl der aktuell am Messquerschnitt vorhandenen
	 * Fahrstreifen abzglich der logisch passivierten Fahrstreifen.
	 * 
	 * @param mq Klasse Messquerschnitt.
	 * @return aktuelle Anzahl der Fahrstreifen des Messquerschnitt. 
	 */
	private int bestimmeAnzFahrstreifenAktuell(MessQuerschnitt mq)
	{
		int anzFs = mq.getAnzahlFahrstreifen();

		if (m_passivierungFahrStreifen != null)
		{
			List<SystemObject> lfs = mq.getFahrstreifen();
			for (int i = 0; i < lfs.size(); i++)
			{
				if (m_passivierungFahrStreifen.isFahrStreifenPassiviert(lfs.get(i)))
				{
					anzFs--;
				}				
			}			
		}
		
		return anzFs;
	}

	/**
	 * Prft, ob der bergebene Fahrstreifen logisch passiviert ist.
	 * 
	 * @param fs Systemobjekt Fahrstreifen.
	 * @return true wenn Fahrstreifen passiviert, ansonsten false;
	 */
	private boolean isFahrstreifenPassiviert(SystemObject fs)
	{
		if (m_passivierungFahrStreifen != null)
		{
			if (m_passivierungFahrStreifen.isFahrStreifenPassiviert(fs))
			{
				return true;
			}
		}
		
		return false;
	}
	/**
	 * Diese Methode bestimmt aus dem bergebenen Ein-/Ausschaltkriterium und dem aktuellen Status
	 * der Ursacheneinheit den neuen Status und gibt ihn zurck.
	 * 
	 * @param einschaltKriterium Status Einschaltkriterium.
	 * @param ausschaltKriterium Status Ausschaltkriterium.
	 * @param statusAkt Aktueller Status der Ursacheneinheit.
	 * @return true wenn Einschaltkriterium erfllt, false wenn Ausschaltkriterium erfllt, ansonsten aktueller Status.
	 */
	private boolean bestimmeKriteriumMq(boolean einschaltKriterium, boolean ausschaltKriterium, boolean statusAkt)
	{
		boolean zurueck = statusAkt;

		// Neuer Status bestimmen
    if (!statusAkt) // UE nicht aktiv
    {
    	if (einschaltKriterium)
    		zurueck = true;
    }
    else
    {
    	if (ausschaltKriterium)
    		zurueck = false;
    }
    
		return zurueck;
	}

	/**
	 * Diese Methode versendet die Analysedaten der Situationsbestimmung ber den Datenverteiler.
	 * 
	 * @param analyse Klasse mit den Analysedaten.
	 */
	private void sendeAnalyseDaten(SituationAnalyseMq analyse)
	{
		DataDescription dd = new DataDescription(m_atgAnalyse, m_aspAnalyse);

		Data data_neu = m_connection.createData(m_atgAnalyse);
		if (data_neu != null)
		{
			data_neu.getScaledValue("Einschaltkriterium").setText((analyse.isEinschaltkriterium())? "Ja" : "Nein"); 
			data_neu.getScaledValue("Ausschaltkriterium").setText((analyse.isAusschaltkriterium())? "Ja" : "Nein"); 
			
			data_neu.getArray("Status").setLength(0);

			for (int i = 0; i < analyse.getLength(); i++)
			{
				data_neu.getArray("Status").setLength(i+1);
				
				Data element = data_neu.getArray("Status").getItem(i);
				if (element != null)
				{
					element.getReferenceValue("MessQuerschnitt").setSystemObject(analyse.getItem(i).getMessQuerschnitt());				
					element.getUnscaledValue("Aktiv").setText(analyse.getItem(i).getAktiv());
				}			
			}
		}

		ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
		
		try
		{
			m_connection.sendData(erg);
		}
		catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		}
		catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	public void update(ResultData results[])
	{		
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
    	// Datensatz Parameter empfangen
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameter))
    	{ 
    		System.out.println("-------> " + resultData);
    		
    		if (m_parameter == null)
    		{
    			m_parameter = new ParameterLkwUeberholverbotSichtWochentag(resultData);
    		}
    		else
    		{
    			m_parameter.setdaten(resultData);
    		}
     		
    		erzeugeObjekteOnlineDaten();
    	}
    	
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterMqZuordnung))
    	{ 
    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterMqZuordnung == null)
    		{
    			m_parameterMqZuordnung = new ParameterMqZuordnung(resultData);
    		}
    		else
    		{
    			m_mqZuordnungAlt.clear();
    			m_mqZuordnungAlt.addAll(m_parameterMqZuordnung.getMessQuerschnitte());

    			m_parameterMqZuordnung.setdaten(resultData);
    		}
     	
    		erzeugeObjekteMessQuerschnitt();
    	}

    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameterUfdsZuordnung))
    	{ 
    		System.out.println("-------> " + resultData);
    		
    		if (m_parameterUfdsZuordnung == null)
    		{
    			m_parameterUfdsZuordnung = new ParameterUfdsZuordnung(resultData);
    		}
    		else
    		{
    			m_ufdsZuordnungAlt.clear();
    			m_ufdsZuordnungAlt.addAll(m_parameterUfdsZuordnung.getUmfeldDatenSensoren());

    			m_parameterUfdsZuordnung.setdaten(resultData);
    		}
     	
    		erzeugeObjekteUmfeldDatenSensoren();
    	}
		}
	}
	
	/**
	 * Diese Methode erzeugt aus der Liste der zugeordenten Umfelddatensensoren die Objekte
	 * fr die Verwaltung der Onlinedaten.
	 * 
	 */
	private void erzeugeObjekteUmfeldDatenSensoren()
	{
		if (m_parameterUfdsZuordnung != null)
		{
			List<SystemObject> listeUfds = m_parameterUfdsZuordnung.getUmfeldDatenSensoren();
			if (listeUfds != null)
			{
				for (int i = 0; i < listeUfds.size(); i++)
				{
					SystemObject obj = listeUfds.get(i);
					
					UmfeldDatenSensor ufdms = new UmfeldDatenSensor(m_connection, obj);
					if (ufdms != null)
					{
						ufdms.datenDavAnmelden();
						
						m_mapUmfeldDatenSensor.put(obj.getPid(), ufdms);
					}
				}
			}
		
			// Nicht mehr bentigte Umfelddatenmessstellen aus Liste entfernen
			for (int i = 0; i < m_ufdsZuordnungAlt.size(); i++)
			{
				SystemObject ufdsAlt = m_ufdsZuordnungAlt.get(i);
				if (!listeUfds.contains(ufdsAlt))
				{
					if (m_mapUmfeldDatenSensor.containsKey(ufdsAlt.getPid()))
					{
						UmfeldDatenSensor _ufds = m_mapUmfeldDatenSensor.get(ufdsAlt.getPid());
						if (_ufds != null)
						{
							System.out.println(ufdsAlt.getPid() + " Daten Abmelden und aus Liste lschen");
							
							_ufds.datenDavAbmelden();
							m_mapUmfeldDatenSensor.remove(ufdsAlt.getPid());
						}
					}
				}
			}
		
		}		
	}

	/**
	 * Diese Methode erzeugt aus der Liste der zugeordenten Messquerschnitte die Objekte
	 * fr die Verwaltung der Onlinedaten.
	 */
	private void erzeugeObjekteMessQuerschnitt()
	{
		if (m_parameterMqZuordnung != null)
		{
			List<SystemObject> listeMq = m_parameterMqZuordnung.getMessQuerschnitte();
			if (listeMq != null)
			{
				for (int i = 0; i < listeMq.size(); i++)
				{
					SystemObject obj = listeMq.get(i);
											
					if (!m_mapMessQuerschnitte.containsKey(obj.getPid()))
					{
						MessQuerschnitt mq = new MessQuerschnitt(m_connection, obj, true, m_aspektPrognose);
						if (mq != null)
						{
							mq.datenDavAnmelden();
							m_mapMessQuerschnitte.put(obj.getPid(), mq);
						}
					}
				}
			}

			// Nicht mehr bentigte Messquerschnitte aus Liste entfernen
			for (int i = 0; i < m_mqZuordnungAlt.size(); i++)
			{
				SystemObject mqAlt = m_mqZuordnungAlt.get(i);
				if (!listeMq.contains(mqAlt))
				{
					if (m_mapMessQuerschnitte.containsKey(mqAlt.getPid()))
					{
						MessQuerschnitt _mq = m_mapMessQuerschnitte.get(mqAlt.getPid());
						if (_mq != null)
						{
							System.out.println(mqAlt.getPid() + " Daten Abmelden und aus Liste lschen");
							
							_mq.datenDavAbmelden();
							m_mapMessQuerschnitte.remove(mqAlt.getPid());
						}
					}
				}
			}
		}
	}
	
	/**
	 * Diese Methode erzeugt aus dem empfangenen Parameter die Objekte
	 * fr die Verwaltung der Onlinedaten.
	 */
	private void erzeugeObjekteOnlineDaten()
	{
		if (m_parameter != null)
		{
			synchronized (m_mapEreignis)
			{
				m_mapEreignis.clear();
				m_fehlerEreignis = false;
				
				/**
				 * Ereignisse
				 */			
				SystemObject etyp = m_parameter.getEreignisTyp();
				if (etyp != null)
				{
					// Alle Ereignisse aus dem Modell holen
					
					List<SystemObject> liste = m_dataModel.getType("typ.ereignis").getElements();
					if (liste != null)
					{
						for (int i = 0; i < liste.size(); i++)
						{
							SystemObject ereignis = liste.get(i);
	
							if (!ereignis.isValid())
								continue;
	
							SystemObject ereignisTyp =  bestimmeEreignisTyp(ereignis);
							if (ereignisTyp != null)
							{
								if (ereignisTyp.equals(etyp))
								{
									if (!m_mapEreignis.containsKey(ereignis.getPid()))
									{
										m_mapEreignis.put(ereignis.getPid(), new Ereignis(m_connection, ereignis));
										System.out.println("Ereignis " + ereignis.getPid() + " ID " + ereignis.getId() + " eingetragen");
									}
								}
							}
						}
					}
					
					if (m_mapEreignis.isEmpty())
					{
						m_fehlerEreignis = true;

						System.out.println("Fehler: Zu EreignisTyp='" + etyp +"' kein Ereignis gefunden!" );
						m_debug.error("Fehler: Zu EreignisTyp='" + etyp +"' kein Ereignis gefunden!" );
					}
				}		
			}
		}
	}

	/**
	 * Prft, ob in der Liste der zu bercksichtigenden Ereignisse ein Ereignis zeitlich gltig ist.
	 * 
	 * @return true wenn ein Ereignis gltig ist, ansonsten false.
	 */
	private boolean isEreignisGltig()
	{
		synchronized (m_mapEreignis)
		{
			if (m_mapEreignis != null)
			{
				if (!m_mapEreignis.isEmpty())
				{
					Object[] keys = m_mapEreignis.keySet().toArray();		  	  	
					for (int i = 0; i < keys.length; i++)
					{
						String key = (String)keys[i];
						
						Ereignis ereignis = m_mapEreignis.get(key);
						if (ereignis != null)
						{
							if (ereignis.isZeitlichGueltig())
								return true;
						}
					}
					
					return false;
				}
			}
		}
		
		return true;
	}

	/**
	 * Bestimmt den Ereignistyp des bergebenen Ereignisses.
	 * 
	 * @return Objekt Ereignistyp.
	 */
	private SystemObject bestimmeEreignisTyp(SystemObject objekt)
	{
		if (objekt != null)
		{
			Data data = objekt.getConfigurationData(m_dataModel.getAttributeGroup("atg.ereignisEigenschaften"));
			if (data != null)
			{
				return data.getReferenceValue("EreignisTypReferenz").getSystemObject();
			}
		}
		return null;
	}
	
	/**
	 * bernahme der bergebenen Daten bzgl. der Passivierten Fahrstreifen in die Klassenvariable <code>m_passivierungFahrStreifen</code>.
	 *  
	 * @param rdata Datensatz <code>atg.sswSbaPassivierungFahrStreifen</code>
	 */
	public void updatePassivierungFahrstreifen(ResultData rdata)
	{
		if (m_passivierungFahrStreifen == null)
			m_passivierungFahrStreifen = new PassivierungFahrstreifen(rdata);
		else
			m_passivierungFahrStreifen.setdaten(rdata);
		
//		System.out.println(m_objekt.getPid() + " PassivierungFahrstreien " + m_passivierungFahrStreifen);
	}

}
